import sys
import os
modules_path = '../../../modules/'
if os.path.exists(modules_path):
    sys.path.insert(1, modules_path)

import json
from datetime import datetime, date
import matplotlib.pyplot as plt
from db.repository import Repository
from db.entities.timeslice import TimeSlice
from db.entities import ClusterSet
from typing import Tuple, Dict

repo = Repository()


def convert_to_time_slice_key(timestamp: str) -> Tuple[int, int]:
    '''Returns the tuple (year, week_of_year) from a timestamp.'''
    timestamp = datetime.fromtimestamp(float(timestamp[0:10]))
    (y, w, _) = timestamp.isocalendar()
    return (y, w)


def get_clusterset():
    clusterset = repo.get_clusterset('Destination_Layer')
    # with open('clustering_results/optics/clusterset_Destination_Layer.txt') as file:
    #     clusterset = ClusterSet(cluster_set_dict=json.loads(file.read()))
    return clusterset


def plt_show_circles(keys, time_slices, cluster_no):
    for k in keys:
        slice_ = time_slices[k]

        if cluster_no in slice_.nodes:
            nodes = slice_.nodes[cluster_no] 
        else:
            nodes = []    
            
        # print(f"{slice_.time} number elements for cluster {cluster_no}: {len(nodes)}")

        plt.title(str(k))

        plt.scatter([n['Longitude_Destination'] for n in nodes],
                    [n['Latitude_Destination'] for n in nodes],
                    s=[len(nodes)*100]*len(nodes))

        plt.pause(0.5)


def plt_show_bars(keys, time_slices, cluster_no):
    x_axis_label_stepsize = 10

    nodes_per_slice_for_single_cluster = \
            [len(time_slices[k].get_nodes_for_cluster(cluster_no))
            for k 
            in keys]

    fig, ax = plt.subplots()
    ax.bar(x=range(len(keys)),
        height=nodes_per_slice_for_single_cluster)

    ax.set_ylabel('Size')
    ax.set_title(f'Cluster-{cluster_no} size over time')
    ax.set_xticks(range(len(keys))[::x_axis_label_stepsize])
    ax.set_xticklabels(keys[::x_axis_label_stepsize])

    plt.show()



clusterset = get_clusterset()


cnt = 0
time_slices = {} 
# for clusterset in clustersets:
for cluster_no in clusterset.clusters:
    for node in cluster_no.nodes:
        # assign the nodes to time slices and recreate the clusters there
        time_key = convert_to_time_slice_key(str(node['Finished_time']))
        
        if time_key not in time_slices:
            time_slices[time_key] = TimeSlice(time_key, clusterset.layer_name)

        time_slices[time_key].add_node_to_cluster(cluster_no.cluster_label, node)
        

# sort chronologically
keys = list(time_slices.keys())
keys.sort()


repo.remove_all_time_slices()
for k,v in time_slices.items():
    repo.add_time_slice(v)


print(len(time_slices))
plt_show_bars(keys, time_slices, cluster_no = 0)
